﻿// The sole owner of this code is Dundas Data Visualization Inc.
// Copyright (c) 2014 Dundas Data Visualization Inc.
// All rights reserved.

using Dundas.BI.Data.DataProviders.Special;
using System;
using System.Collections.Generic;
using System.Globalization;
using System.IO;
using System.Linq;
using System.Net;
using System.Text;
using System.Threading.Tasks;
using System.Web.Script.Serialization;

namespace MyCompany.Sample.TwitterDataProvider
{
	public class TwitterConnection : ConnectionBase
	{
		#region Fields

		private TwitterConnectionInfo twitterConnectionInfo;
        internal const string dateTimeFormat = "ddd MMM dd HH:mm:ss zzzz yyyy";

		#endregion Fields

		#region Constructors

		public TwitterConnection(TwitterConnectionInfo twitterConnectionInfo)
		{
			this.twitterConnectionInfo = twitterConnectionInfo;
		}

		#endregion Constructors

		#region Internal Methods

        /// <summary>
        /// Gets the user timeline.
        /// </summary>
        /// <param name="screenName">The users twitter screen name.</param>
        /// <returns>A list of Twitter </returns>
		internal IEnumerable<TwitterData> GetUserTimeline(string screenName)
		{

            HttpWebRequest apiRequest =
                (HttpWebRequest)WebRequest.Create(
                    string.Concat(
                        "https://api.twitter.com/1.1/statuses/user_timeline.json?screen_name=",
                        screenName,
                        "&amp;include_rts=false&amp;exclude_replies=true&amp;"
                        )
                    );

			apiRequest.Headers.Add(
				OAuthHelper.GetAuthorizationHeaders(twitterConnectionInfo)
				);

			apiRequest.Method = "Get";

			using (WebResponse timeLineResponse = apiRequest.GetResponse())
			{
				using (var reader = new StreamReader(timeLineResponse.GetResponseStream()))
				{

					JavaScriptSerializer javaScriptSerializer = new JavaScriptSerializer();
					return
							javaScriptSerializer.Deserialize<TwitterData[]>(
								reader.ReadToEnd()
							);

				}
			}
		}

        internal IEnumerable<TwitterData> GetUserTimelineForCurrentYear(string screenName)
        {


            List<TwitterData> fullTwitterDataObjects = new List<TwitterData>();

            int currentYear = DateTime.Now.Year;

            bool keepRequesting = true;
           
            string lastEntryProcessed = string.Empty;

            while(keepRequesting)
            {

                string url =
                     string.Empty;

                if(string.IsNullOrEmpty(lastEntryProcessed))
                {
                    url =
                        string.Concat(
                            "https://api.twitter.com/1.1/statuses/user_timeline.json?screen_name=",
                            screenName,
                            "&amp;include_rts=true&amp;exclude_replies=true&amp;count=200&amp;"
                            );
                }
                else
                {
                    url =
                    string.Concat(
                        "https://api.twitter.com/1.1/statuses/user_timeline.json?screen_name=",
                        screenName,
                        "&amp;include_rts=true&amp;exclude_replies=true&amp;max_id=", lastEntryProcessed, "&amp;count=200&amp;"
                        );
                }

                HttpWebRequest apiRequest =
                    (HttpWebRequest)WebRequest.Create(
                        url
                        );

                apiRequest.Headers.Add(
                    OAuthHelper.GetAuthorizationHeaders(twitterConnectionInfo)
                    );

                apiRequest.Method = "Get";

                using (WebResponse timeLineResponse = apiRequest.GetResponse())
                {
                    using (var reader = new StreamReader(timeLineResponse.GetResponseStream()))
                    {

                        JavaScriptSerializer javaScriptSerializer = new JavaScriptSerializer();

                        List<TwitterData> twitterDataObjects = 
                            javaScriptSerializer.Deserialize<List<TwitterData>>(
                                    reader.ReadToEnd()
                                );


                        if(twitterDataObjects.Count() == 0 )
                        {
                            keepRequesting = false;
                        }

                        foreach(TwitterData data in twitterDataObjects)
                        {
                            // Filter out retweets.  
                            if(!data.text.StartsWith("RT @"))
                            {
                                if (!data.id_str.Equals(lastEntryProcessed))
                                {

                                    DateTime dataDate =
                                            DateTime.ParseExact(
                                            data.created_at,
                                            TwitterConnection.dateTimeFormat,
                                            CultureInfo.InvariantCulture
                                        );

                                    if (dataDate.Year.Equals(currentYear))
                                    {
                                        fullTwitterDataObjects.Add(data);
                                    }
                                    else
                                    {
                                        keepRequesting = false;
                                    }

                                }
                            }
                        }

                        lastEntryProcessed = twitterDataObjects.Last().id_str;


                            
                    }
                }
           
            }

            return fullTwitterDataObjects;
        }

		#endregion Internal Methods

		#region Public Methods

        /// <summary>
        /// Closes the connection to the database.
        /// </summary>
        /// <remarks>
        /// In this sample we do not have a need to close a connection.
        /// </remarks>
		public override void Close()
		{
		}

        /// <summary>
        /// Opens a database connection with the settings specified by the ConnectionString
        /// property of the provider-specific Connection object.
        /// </summary>
        /// <remarks>
        /// In this sample we do not have a need to open connection.
        /// </remarks>
		public override void Open()
		{
		}

		#endregion Public Methods
	}
}