﻿
// The sole owner of this code is Dundas Data Visualization Inc.
// Copyright (c) 2014 Dundas Data Visualization Inc.
// All rights reserved.

using Dundas.BI.Data;
using Dundas.BI.Data.DataProviders;
using Dundas.BI.Data.DataProviders.Special;
using Dundas.BI.Data.Metadata;
using Dundas.BI.Data.Metadata.Relational;
using Dundas.BI.Data.Parameters;

using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace MyCompany.Sample.TwitterDataProvider
{
    /// <summary>
    /// This represents a Twitter Data Provider
    /// </summary>
    public class TwitterDataProvider : TabularProvider
    {
        #region Fields

        private IReadOnlyList<Dundas.BI.Data.DataProviders.ProviderProperty> _propertyDescriptors;

        #endregion Fields

        #region Public Properties

        /// <summary>
        /// Gets the standard component description.
        /// </summary>
        public override string ComponentDescription
        {
            get { return "This data provider returns data from twitter"; }
        }
            
        /// <summary>
        /// Gets the component ID.
        /// </summary>
        public override Guid ComponentId
        {
            get { return new Guid("5202d55c-3051-4a5d-8b6f-1c686bb1bb40"); }
        }

        /// <summary>
        /// Gets the standard component name.
        /// </summary>
        public override string ComponentName
        {
            get { return "Twitter Data Provider"; }
        }

        /// <summary>
        /// Gets the data provider specific property descriptors.
        /// </summary>
        public override IReadOnlyList<Dundas.BI.Data.DataProviders.ProviderProperty> PropertyDescriptors
        {
            get
            {
                if (_propertyDescriptors == null)
                {
                    _propertyDescriptors = DefineProperties();
                }

                return _propertyDescriptors;
            }
        }

        /// <summary>
        /// Gets the data provider specific descriptors for tabular structure settings.
        /// </summary>
        public override IReadOnlyList<Dundas.BI.Data.DataProviders.ProviderProperty> StructureDescriptors
        {
            get
            {
                List<ProviderProperty> properties = new List<ProviderProperty>();
                return new ReadOnlyCollection<ProviderProperty>(properties);
            }
        }

        #endregion Public Properties

        #region Private Methods

        /// <summary>
        /// Connects the specified data connector.
        /// </summary>
        /// <param name="dataConnector">The data connector.</param>
        /// <returns>A <see cref="T:Dundas.BI.Sample.TwitterDataProvider.TwitterConnection"/>.</returns>
        /// <exception cref="System.ArgumentException">Missing required value.</exception>
        private static TwitterConnection Connect(IProviderConnector dataConnector)
        {


            string consumerKey =
                dataConnector.ProviderValues.GetStringValue(TwitterPropertyIds.ConsumerKeyId, string.Empty);

            string consumerSecret =
                dataConnector.ProviderValues.GetStringValue(TwitterPropertyIds.ConsumerSecretId, string.Empty);


            if (string.IsNullOrEmpty(consumerKey) ||
                string.IsNullOrEmpty(consumerSecret)
                )
            {
                throw new ArgumentException("Missing required value.");
            }

            TwitterConnectionInfo twitterConnectionInfo = new TwitterConnectionInfo(consumerKey, consumerSecret);

            return new TwitterConnection(twitterConnectionInfo);
        }

        /// <summary>
        /// Defines the properties for the data provider.
        /// </summary>
        /// <returns></returns>
        private IReadOnlyList<Dundas.BI.Data.DataProviders.ProviderProperty> DefineProperties()
        {
            List<ProviderProperty> properties = new List<ProviderProperty>();
            ProviderProperty propertyDescriptor;

            propertyDescriptor = new StringProperty(
                TwitterPropertyIds.ConsumerKeyId,
                "Consumer Key Id",
                "Consumer Key Description",
                string.Empty, int.MaxValue);
            propertyDescriptor.IsEncryptionRequired = true;
            properties.Add(propertyDescriptor);

            propertyDescriptor = new StringProperty(
                TwitterPropertyIds.ConsumerSecretId,
                "Consumer Secret Id",
                "Consumer Secret Description",
                string.Empty, int.MaxValue);
            propertyDescriptor.IsEncryptionRequired = true;
            properties.Add(propertyDescriptor);

            return new ReadOnlyCollection<ProviderProperty>(properties);
        }

        #endregion Private Methods

        #region Protected Methods

        /// <summary>
        /// Creates the reader for the specified query object.
        /// </summary>
        /// <param name="connection">Opened connection.</param>
        /// <param name="query">The query definition.</param>
        /// <returns>
        /// The reader for the specified query object.
        /// </returns>
        protected override Dundas.BI.Data.DataProviders.Reader CreateReader(IDisposable connection, Dundas.BI.Data.DataProviders.QueryDefinition query)
        {
            return new TwitterDataReader((TwitterConnection)connection, query);
        }

        #endregion Protected Methods

        #region Public Methods

        /// <summary>
        /// Builds the connection string.
        /// </summary>
        /// <param name="dataConnector">The data connector.</param>
        /// <returns>
        /// The connection string.
        /// </returns>
        public override string BuildConnectionString(Dundas.BI.Data.DataProviders.IProviderConnector dataConnector)
        {
            return string.Empty;
        }

        /// <summary>
        /// This method is to discover the data structure.  Columns and Parameters will be defined in the object.
        /// </summary>
        /// <param name="connection">The connection.</param>
        /// <param name="dataStructure"></param>
        /// <returns>A Datastructure with columns defined, and parameters defined.</returns>
        public override DataStructure DiscoverStructure(IDisposable connection, DataStructure dataStructure)
        {
            TwitterConnection twitterConnection = connection as TwitterConnection;

            RelationalStructure structure = dataStructure as RelationalStructure;

            structure.Columns.Clear();

            structure.Columns.Add(new RelationalColumn(structure, "ScreenName", DataType.String(16)));
            structure.Columns.Add(new RelationalColumn(structure, "Text", DataType.String(165)));
            structure.Columns.Add(new RelationalColumn(structure, "CreatedAt", new DataType(DatabaseType.DateTime)));
            structure.Columns.Add(new RelationalColumn(structure, "RetweetCount", new DataType(DatabaseType.Int32)));
            structure.Columns.Add(new RelationalColumn(structure, "FavoriteCount", new DataType(DatabaseType.Int32)));

            NativeParameter nativeParameter =
                new NativeParameter("ScreenName", DataType.String(260));

            nativeParameter.IsOptional = false;
            nativeParameter.Description = "Twitter Screen Name";
            nativeParameter.Direction = NativeParameterDirection.Input;

            structure.Parameters.Clear();

            structure.Parameters.Add(nativeParameter);

            return structure;
        }

        /// <summary>
        /// Gets the list of the structures with the specified type.
        /// </summary>
        /// <param name="connection">The connection.</param>
        /// <param name="structureTypes">The structure types.</param>
        /// <returns>
        /// The list of the structures.
        /// </returns>
        public override IEnumerable<Dundas.BI.Data.Metadata.DataStructure> DiscoverStructures(IDisposable connection, Dundas.BI.Data.Metadata.SupportedDataStructures structureTypes)
        {
            List<DataStructure> dataStructures = new List<DataStructure>();

            DataStructure dataStructure = new RelationalStructure(SupportedDataStructures.RelationalStoredProcedure, "UserTimeline");

            dataStructures.Add(dataStructure);

            return dataStructures;
        }

        /// <summary>
        /// Gets the open connection.
        /// </summary>
        /// <param name="dataConnector">The data connector.</param>
        /// <returns>
        /// The open connection.
        /// </returns>
        /// <exception cref="System.ArgumentNullException">dataConnector</exception>
        public override IDisposable GetConnection(Dundas.BI.Data.DataProviders.IProviderConnector dataConnector)
        {
            if (dataConnector == null)
            {
                throw new ArgumentNullException("dataConnector");
            }

            return TwitterDataProvider.Connect(dataConnector);
        }

        /// <summary>
        /// Gets the ID of the credentials provider.
        /// </summary>
        /// <param name="dataConnector">The data connector.</param>
        /// <returns>
        /// The ID of the credentials provider.
        /// </returns>
        public override Guid GetCredentialsProviderId(Dundas.BI.Data.DataProviders.IProviderConnector dataConnector)
        {
            return Guid.Empty;
        }

        /// <summary>
        /// Determines whether third party driver required for this data provider is installed.
        /// </summary>
        /// <returns>
        ///   <see langword="true" /> if the third party driver detected, otherwise <see langword="false" />.
        /// </returns>
        public override bool IsDriverInstalled()
        {
            return true;
        }

        /// <summary>
        /// Tests the connection.
        /// </summary>
        /// <param name="dataConnector">The data connector.</param>
        /// <returns>
        /// The test connection result.
        /// </returns>
        public override Dundas.BI.Data.DataProviders.TestConnectionResult TestConnection(Dundas.BI.Data.DataProviders.IProviderConnector dataConnector)
        {
            using (TwitterConnection twitterConnection = TwitterDataProvider.Connect(dataConnector))
            {

                twitterConnection.GetUserTimeline("DundasData");

                TestConnectionResult result = new TestConnectionResult(null, null);
                return result;

            }
        }

        /// <summary>
        /// Updates from connection string.
        /// </summary>
        /// <param name="dataConnector">The data connector.</param>
        /// <param name="connectionString">The connection string.</param>
        public override void UpdateFromConnectionString(Dundas.BI.Data.DataProviders.IProviderConnector dataConnector, string connectionString)
        {
            return;
        }

        #endregion Public Methods
    }
}