﻿#region Header

// The sole owner of this code is Dundas Data Visualization Inc.
// Copyright (c) 2014 Dundas Data Visualization Inc.
// All rights reserved.

#endregion Header

using Dundas.BI.Data.DataProviders;
using Dundas.BI.Data.Metadata;
using Dundas.BI.Data.Metadata.Relational;

using System;
using System.Collections.Generic;
using System.Globalization;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Web.Script.Serialization;

namespace MyCompany.Sample.TwitterDataProvider
{
    /// <summary>
    /// This class represents a Twitter Data Reader.
    /// </summary>
	public class TwitterDataReader : Reader
	{
		#region Fields

		private List<TwitterData> twitterDataList;
		private TwitterData _currentItem;
		private int _currentItemCount;
		private bool _disposed;
		private QueryDefinition _queryDefinition;
		private RelationalStructure _relationalStructure;
		private TwitterConnection _twitterConnection;

		#endregion Fields

		#region Constructors

        /// <summary>
        /// Initializes a new instance of the <see cref="TwitterDataReader"/> class.
        /// </summary>
        /// <param name="twitterConnection">The twitter connection.</param>
        /// <param name="query">The query.</param>
        public TwitterDataReader(TwitterConnection twitterConnection, QueryDefinition query)
        {
	        _currentItemCount = 0;

	        _twitterConnection = twitterConnection;
	        _queryDefinition = query;


	        _relationalStructure = (RelationalStructure)query.StructureDefinition;
        }

		#endregion Constructors

		#region Public Properties

        /// <summary>
        /// Gets the column metadata of the reader.
        /// </summary>
        public override IList<Dundas.BI.Data.Metadata.IDataElement> Schema
        {
	        get
	        {
		        return this._relationalStructure.DataElements.ToList();
	        }
        }

        /// <summary>
        /// Gets the element value located at the specified index.
        /// </summary>
        /// <value>
        /// The element value.
        /// </value>
        /// <param name="elementIndex">Element index.</param>
        /// <returns>The element value.</returns>
        public override object this[int elementIndex]
        {
	        get
	        {
		        object returnObject;

		        switch (elementIndex)
		        {
			        case 0:
				        returnObject =
					        _currentItem.user.screen_name;
				        break;
			        case 1:
				        returnObject =
					        _currentItem.text;
				        break;
			        case 2:
				        DateTime dataDate =
					        DateTime.ParseExact(
					        _currentItem.created_at,
					        TwitterConnection.dateTimeFormat,
					        CultureInfo.InvariantCulture
				        );
				        returnObject =
					        dataDate;
				        break;
			        case 3:
				        returnObject = _currentItem.retweet_count;
				        break;
			        case 4:
				        returnObject = _currentItem.favorite_count;
				        break;
			        default:
				        returnObject = null;
				        break;

		        }

		        return returnObject;
	        }
        }

		#endregion Public Properties

		#region Private Methods

        /// <summary>
        /// Gets the next item from the twitter data list.
        /// </summary>
        /// <returns>The next Twitter Data object from the Twitter data list, or <see langword="null" />.</returns>
		private TwitterData NextItem()
		{
			if (twitterDataList.Count() > _currentItemCount)
			{
				return
					twitterDataList[_currentItemCount++];
			}
			else
			{
				return null;

			}
		}

		#endregion Private Methods

		#region Protected Methods

        /// <summary>
        /// Releases unmanaged and (optionally) managed resources.
        /// </summary>
        /// <param name="disposing"><see langword="true" /> to release both managed and unmanaged resources; <see langword="false" /> to release only unmanaged resources.</param>
		protected override void Dispose(bool disposing)
		{

			if (this._disposed)
			{
				return;
			}

			this._disposed = true;
		}

		#endregion Protected Methods

		#region Public Methods

        /// <summary>
        /// Cancels the execution of the reader.
        /// </summary>
		public override void Cancel()
		{
		}

        /// <summary>
        /// Advances the reader to the next record.
        /// </summary>
        /// <returns><see langword="true"/> if there are more rows; otherwise, <see langword="false"/>.</returns>
        public override bool Read()
        {
            // System.Diagnostics.Debugger.Launch();

	        if(twitterDataList == null)
	        {
		        string screenName =
			        this._queryDefinition.Parameters.FirstOrDefault(
				        parameter => parameter.Name == "ScreenName"
				        ).Value.ToString();

		        twitterDataList =
			        this._twitterConnection.GetUserTimelineForCurrentYear(screenName).ToList();
	        }

	        _currentItem = NextItem();

	        if(_currentItem == null)
	        {
		        return false;
	        }
	        else
	        {
		        return true;
	        }
        }

		#endregion Public Methods
	}
}